/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect } from "react";
import moment from 'moment';
// Maerial ui components
import Card from "@material-ui/core/Card";
import CardContent from "@material-ui/core/CardContent";
import Typography from "@material-ui/core/Typography";
import Button from "@material-ui/core/Button";
import ListRoundedIcon from "@material-ui/icons/ListRounded";
import ButtonGroup from "@material-ui/core/ButtonGroup";
import { useStyles } from "../../Styles/DataCard";
import Grid from '@material-ui/core/Grid';
// Icons
import EditRoundedIcon from "@material-ui/icons/EditRounded";
//imports from local files
import { GetAData } from "../../../Filters";
import { Link } from 'react-router-dom'
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import { getRequest } from '../../../../Components/HelperFunctions/getAxiosConfig';

const CollectionCard = ({ match }) => {
    console.log(match);
    const classes = useStyles();

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [item, setItem] = useState({}); //Necessary to consult the API, data

    const DisplayDate = (date) => {
        const convertedData = moment.utc(date);
        return moment(convertedData)
            .format("LLL")
            .toString();
    };

    useEffect(() => {
        getRequest(
            GetAData("collections", match.params.id),
            (data, header) => {
                setItem(data);
                setIsLoaded(true);
                setError(false);
            },
            (error) => {
                setIsLoaded(true);
                setError(true);
            }
        )
    }, []);

    if (error) {
        return <div>Houve um erro</div>;
    } else if (!isLoaded) {
        return <LoadingSpinner text="Carregando..." />
    } else {
        const DATA = [
            {
                subTitle: "ID",
                prop: item.id,
            },
            {
                subTitle: "Nome",
                prop: item.name,
            },
            {
                subTitle: "Descrição",
                prop: item.description,
            },
            {
                subTitle: "Endereço",
                prop: item.owner ? item.owner.name : null,
            },
            {
                subTitle: "Score",
                prop: item.score,
            },
            {
                subTitle: "Privacidade",
                prop: item.privacy,
            },
            {
                subTitle: "Criação",
                prop: DisplayDate(item.created_at),
            },
            {
                subTitle: "Atualização",
                prop: DisplayDate(item.updated_at),
            },
        ];

        return (
            <Card>
                <CardContent>
                    <Grid xs={12} justify="space-between" alignItems="center" container>
                        <Grid item>
                            <Typography className={classes.title} color="inherit" gutterBottom>
                                {item.name}
                            </Typography>
                        </Grid>
                        <Grid item>
                            <ButtonGroup
                                color="primary"
                                aria-label="outlined primary button group"
                            >
                                <Link style={{ textDecoration: 'none' }} to={`/admin/Collections`}>
                                    <Button
                                        startIcon={<ListRoundedIcon />}
                                        color="primary"
                                        variant="outlined"
                                    >
                                        Listar
                                    </Button>
                                </Link>

                                <Link style={{ textDecoration: 'none' }} to={`/admin/EditCollection/${item.id}`}>
                                    <Button
                                        startIcon={<EditRoundedIcon />}
                                        color="primary"
                                        variant="outlined"
                                    >
                                        Editar
                                    </Button>
                                </Link>
                            </ButtonGroup>
                        </Grid>
                    </Grid>
                    {DATA.map((info, index) => (
                        <div className={classes.displayColumn} key={index}>
                            <Typography color="initial" className={classes.subTitle}>
                                {info.subTitle}
                            </Typography>
                            <Typography color="textSecondary">
                                {info.prop === null ? "Sem dados" : info.prop}
                            </Typography>
                        </div>
                    ))}
                </CardContent>
            </Card>
        );
    }
};

export default CollectionCard;
