/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect } from "react";
import moment from 'moment';
// Imports from local files
import TableData from "../../../Components/Components/Table";
import SnackBar from "../../../../Components/SnackbarComponent";
import AlertDialog from "../../../Components/Components/AlertDialog";
import LoadingSpinner from '../../../../Components/LoadingSpinner';
// Imports about icon
import FilterListRoundedIcon from "@material-ui/icons/FilterListRounded";
import AddRoundedIcon from "@material-ui/icons/AddRounded";
import UpdateRoundedIcon from "@material-ui/icons/UpdateRounded";
import TableBody from "@material-ui/core/TableBody";
import TableCell from "@material-ui/core/TableCell";
import TableRow from "@material-ui/core/TableRow";
import IconButton from "@material-ui/core/IconButton";
import CancelRoundedIcon from "@material-ui/icons/CancelRounded";
import VisibilityIcon from "@material-ui/icons/Visibility";
import DeleteIcon from "@material-ui/icons/Delete";
// Import from material-ui
import { withStyles, makeStyles } from "@material-ui/core/styles";
import Paper from "@material-ui/core/Paper";
import Button from "@material-ui/core/Button";
import Grid from "@material-ui/core/Grid";
import { Typography, CircularProgress } from "@material-ui/core";
import Popover from "@material-ui/core/Popover";
import TextField from "@material-ui/core/TextField";
// services
import { getRequest, deleteRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
//Filters
import { Url, DeleteFilter } from "../../../Filters";
//router 
import { Link } from 'react-router-dom';

let currPage = 0;

const StyledTableCell = withStyles((theme) => ({
    head: {
        backgroundColor: theme.palette.common.black,
        color: theme.palette.common.white,
    },
    body: {
        fontSize: 14,
    },
}))(TableCell);

const StyledTableRow = withStyles((theme) => ({
    root: {
        "&:nth-of-type(odd)": {
            backgroundColor: theme.palette.action.hover,
        },
    },
}))(TableRow);

const useStyles = makeStyles((theme) => ({
    root: {
        flexGrow: 1,
    },
    paper: {
        padding: theme.spacing(1),
        textAlign: "start",
    },
    button: {
        margin: theme.spacing(1),
        alignSelf: "flex-end",
    },
}));

const EducationalObjects = () => {
    const classes = useStyles();

    const addOndeLenght = [""];

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [items, setItems] = useState([]); //Necessary to consult the API, data

    const [deleteItem, setDeleteItem] = useState({}); //Delete Item
    const [isLoadingToDelete, setIsLoadingToDelete] = useState(null);
    const [isUpdating, setIsUpdating] = useState(false);
    const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false);

    const [openAlertDialog, setOpenAlertDialog] = useState(false);

    const [snackInfo, setSnackInfo] = useState({
        message: "",
        icon: "",
        open: false,
        color: "",
    });

    // **************** About the PopOver Menu ****************
    const [anchorEl, setAnchorEl] = React.useState(null);
    const [showAuthorField, setShowAuthorField] = useState(false); //show the text field of filter by Author
    const [showDescriptionField, setShowDescriptionField] = useState(false); //show the text field of the filter by description
    const [showStandadSearch, setShowStandarSearchField] = useState(false);

    const [search, setSeacrh] = useState("");
    const [author, setAuthor] = useState("");
    const [description, setDescription] = useState(null);

    const AuthorHandler = () => {
        setAuthor("")
        setShowAuthorField(!showAuthorField);
        let url;
        if (description)
            url = Url("learning_objects", `"name" : "${search}", "author" : "", "description" : "${description}"`, currPage, "DESC")
        else
            url = Url("learning_objects", `"name" : "${search}", "author" : ""`, currPage, "DESC")
        Filter(url);
    };

    const DescHandler = () => {
        setDescription(null)
        setShowDescriptionField(!showDescriptionField);
        let url;
        url = Url("learning_objects", `"name" : "${search}", "author" : "${author}"`, currPage, "DESC")
        Filter(url);
    };

    const StandartHandler = () => {
        setSeacrh("")
        setShowStandarSearchField(!showStandadSearch);
        let url;
        if (description)
            url = Url("learning_objects", `"name" : "", "author" : "${author}", "description" : "${description}"`, currPage, "DESC")
        else
            url = Url("learning_objects", `"name" : "", "author" : "${author}"`, currPage, "DESC")
        Filter(url);
    };

    const handleClick = (event) => {
        setAnchorEl(event.currentTarget);
    };

    const handleClose = () => {
        setAnchorEl(null);
    };

    const open = Boolean(anchorEl);
    const id = open ? "simple-popover" : undefined;

    const OnChangeSearchHandler = (e) => {
        setSeacrh(e.target.value);
        let url;
        if (description)
            url = Url("learning_objects", `"name" : "${search}", "author" : "${author}", "description" : "${description}"`, currPage, "DESC")
        else
            url = Url("learning_objects", `"name" : "${search}", "author" : "${author}"`, currPage, "DESC")
        Filter(url);
    };

    const onChangeAuthorHandler = (e) => {
        setAuthor(e.target.value);
        let url;
        if (description)
            url = Url("learning_objects", `"name" : "${search}", "author" : "${author}", "description" : "${description}"`, currPage, "DESC")
        else
            url = Url("learning_objects", `"name" : "${search}", "author" : "${author}"`, currPage, "DESC")
        Filter(url);
    };

    const onChangeDescriptionHandler = (e) => {
        setDescription(e.target.value);
        let url;
        if (description)
            url = Url("learning_objects", `"name" : "${search}", "author" : "${author}", "description" : "${description}"`, currPage, "DESC")
        else
            url = Url("learning_objects", `"name" : "${search}", "author" : "${author}"`, currPage, "DESC")
        Filter(url);
    };
    // **************** About the PopOverMenu ****************

    //Controlls the state of the Alert Dialog
    const HandleStateAlertDialog = (i) => {
        const obj = { ...items[i] };
        setDeleteItem(obj);
        setOpenAlertDialog(!openAlertDialog);
    };

    //Controlls the state and the informations of the snack
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color,
        });
    };

    //Defines which row must show the circular progress
    const HandleStateCircularProgress = (i) => {
        setIsLoadingToDelete(i);
    };

    //Filters the search
    const Filter = (api) => {
        getRequest(
            api,
            (data, header) => {
                const arrData = [...data];
                setItems(arrData.concat(addOndeLenght));
            },
            (error) => {
                setError(true);
            }
        )
    };

    //This function updates List every time the content of the api changes
    const UpdtateListData = (api) => {
        getRequest(
            api,
            (data, header) => {
                setItems([...data.concat(addOndeLenght)]);
                HandleSnack(
                    "A lista de dados foi atualizada",
                    true,
                    "success",
                    "#228B22"
                );
                setIsUpdating(false);
            },
            (error) => {
                HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                setIsUpdating(false);
            }
        )
    };

    //Called when user want to delete one institution
    async function DeleteHandler() {
        const id = deleteItem.id;
        HandleStateAlertDialog(null);
        deleteRequest(
            DeleteFilter("institutions", id),
            (data) => {
                if (data.errors)
                    HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                else {
                    HandleSnack(
                        "A instituição foi deletada com sucesso",
                        true,
                        "success",
                        "#228B22"
                    );
                    currPage = 0;
                    UpdtateListData(Url("institutions", "", `${currPage}`, "DESC"));
                    HandleStateCircularProgress(null);
                }
            },
            (error) => {
                HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                HandleStateCircularProgress(null);
            }
        )
    }

    const LoadMoreItens = async (api) => {
        setIsLoadingMoreItems(true);
        getRequest(
            api,
            (data, header) => {
                const arrData = [...data];
                if (arrData.length === 0) {
                    HandleSnack(
                        "Não há mais dados para serem carregados",
                        true,
                        "warning",
                        "#FFC125"
                    );
                } else {
                    const arrItems = [...items];
                    arrItems.pop(); //Deleting the last position, that was used to display the button of load more items
                    const arrResult = arrItems.concat(arrData);
                    setItems(arrResult.concat(addOndeLenght));
                }
                setIsLoadingMoreItems(false);
            },
            (error) => {
                HandleSnack("Erro ao carregar os dados", true, "warning", "#FA8072");
                setIsLoadingMoreItems(false);
            }
        )
    };

    const DisplayDate = (date) => {
        const convertedData = moment.utc(date);
        return moment(convertedData)
            .format("LLL")
            .toString();
    };

    useEffect(() => {
        getRequest(
            Url("learning_objects", `"name" : "${search}", "author" : "${author}"`, `${currPage}`, "DESC"),
            (data, header) => {
                setIsLoaded(true);
                setItems(data.concat(addOndeLenght));
            },
            (error) => {
                setError(true);

            }
        )
    }, []);

    if (error) {
        return <div>Error: {error.message}</div>;
    }
    if (!isLoaded) {
        return <LoadingSpinner text="Carregando..." />
    } else {
        //Words that defines that column
        const topTable = [
            "CRIADO EM",
            "NOME",
            "DESCRIÇÃO",
            "AUTOR",
            "SCORE",
            "VISUALIZAR",
            "DELETAR",
        ];

        //Buttons from PopOverMenu
        const flatButtonsFromPopOverMenu = [
            {
                label: "Pesquisa padrão",
                onClick: StandartHandler,
                color: showStandadSearch ? "primary" : "default",
            },
            {
                label: "Autor",
                onClick: AuthorHandler,
                color: showAuthorField ? "primary" : "default",
            },
            {
                label: "Descrição",
                onClick: DescHandler,
                color: showDescriptionField ? "primary" : "default",
            },
        ];

        //Field of the Filter
        const TextFieldOfTheFilter = [
            {
                label: "Pesquisar",
                show: showStandadSearch,
                onChange: (event) => OnChangeSearchHandler(event),
                hide: StandartHandler,
            },
            {
                label: "Autor",
                show: showAuthorField,
                onChange: (event) => onChangeAuthorHandler(event),
                hide: AuthorHandler,
            },
            {
                label: "Descrição",
                show: showDescriptionField,
                onChange: (event) => onChangeDescriptionHandler(event),
                hide: DescHandler,
            },
        ];

        return (
            <div>
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() =>
                        setSnackInfo({
                            message: "",
                            icon: "",
                            open: false,
                            color: "",
                        })
                    }
                />
                {/************** Start of the header **************/}
                <Paper style={{ padding: "1em" }}>
                    <Grid container spacing={3} direction="row" alignItems="center">
                        <Grid item xs={6}>
                            <Typography className={classes.paper} variant="h4">
                                Lista de objetos educacionais
                            </Typography>
                        </Grid>
                        <Grid
                            item
                            xs={6}
                        >
                            <Grid container justify="flex-end" spacing={3}>
                                <Grid item>
                                    <Button
                                        aria-describedby={id}
                                        variant="contained"
                                        color="secondary"
                                        className={classes.button}
                                        onClick={(event) => {
                                            handleClick(event)
                                        }}
                                        startIcon={
                                            <FilterListRoundedIcon style={{ fill: "white" }} />
                                        }
                                    >
                                        Filtrar
                                    </Button>
                                    <Popover
                                        id={id}
                                        open={open}
                                        anchorEl={anchorEl}
                                        onClose={handleClose}
                                        anchorOrigin={{
                                            vertical: "bottom",
                                            horizontal: "center",
                                        }}
                                        transformOrigin={{
                                            vertical: "top",
                                            horizontal: "center",
                                        }}
                                    >
                                        {flatButtonsFromPopOverMenu.map((flat, index) => (
                                            <Button
                                                key={index}
                                                onClick={flat.onClick}
                                                color={flat.color}
                                            >
                                                {flat.label}
                                            </Button>
                                        ))}
                                    </Popover>
                                </Grid>

                                <Grid item>
                                    <Button
                                        variant="contained"
                                        color="secondary"
                                        className={classes.button}
                                        startIcon={<UpdateRoundedIcon />}
                                        disabled={isUpdating}
                                        onClick={() => {
                                            currPage = 0;
                                            setIsUpdating(true);
                                            if (description)
                                                UpdtateListData(Url("learning_objects", `"name" : "${search}", "author" : "${author}", "description" : "${description}"`, currPage, "DESC"))
                                            else
                                                UpdtateListData(Url("learning_objects", `"name" : "${search}", "author" : "${author}"`, currPage, "DESC"))
                                        }}
                                    >
                                        {isUpdating ? <CircularProgress size={24} /> : "Atualizar"}
                                    </Button>
                                </Grid>
                            </Grid>
                        </Grid>
                    </Grid>
                    <Grid item xs={12}>
                        <Grid container justify="space-between" spacing={3}>
                            {TextFieldOfTheFilter.map((field, index) => (
                                <Grid item key={index}>
                                    {field.show ? (
                                        <div>
                                            <TextField
                                                id={index}
                                                label={field.label}
                                                type="search"
                                                onChange={field.onChange}
                                            />
                                            <IconButton
                                                size="small"
                                                color="primary"
                                                onClick={field.hide}
                                            >
                                                <CancelRoundedIcon />
                                            </IconButton>
                                        </div>
                                    ) : null}
                                </Grid>
                            ))}
                        </Grid>
                    </Grid>
                </Paper>
                {/************** End of the header **************/}

                <div style={{ height: "2em" }}></div>

                {/************** Start of display data in table **************/}
                <TableData top={topTable}>
                    <TableBody>
                        {items.map((row, index) =>
                            index === items.length - 1 ? (
                                <StyledTableRow key={index}>
                                    {/* Button to load more data */}
                                    <StyledTableCell>
                                        <Button
                                            color="primary"
                                            variant="text"
                                            startIcon={<AddRoundedIcon />}
                                            disabled={
                                                isLoadingMoreItems
                                            }
                                            onClick={() => {
                                                currPage++
                                                if (description)
                                                    LoadMoreItens(Url("learning_objects", `"name" : "${search}", "author" : "${author}", "description" : "${description}"`, currPage, "DESC"))
                                                else
                                                    LoadMoreItens(Url("learning_objects", `"name" : "${search}", "author" : "${author}"`, currPage, "DESC"))
                                            }}
                                        >
                                            {isLoadingMoreItems ? (
                                                <CircularProgress size={24} />
                                            ) : (
                                                    "Carregar mais itens"
                                                )}
                                        </Button>
                                    </StyledTableCell>
                                </StyledTableRow>
                            ) : (
                                    <StyledTableRow key={index}>
                                        <StyledTableCell component="th" scope="row">
                                            {DisplayDate(row.created_at)}
                                        </StyledTableCell>
                                        <StyledTableCell align="right">
                                            {row.name}
                                        </StyledTableCell>
                                        <StyledTableCell align="right">
                                            {row.description}
                                        </StyledTableCell>
                                        <StyledTableCell align="right">
                                            {row.author}
                                        </StyledTableCell>
                                        <StyledTableCell align="right">
                                            {row.score}
                                        </StyledTableCell>
                                        <StyledTableCell align="right">
                                            <Link to={`/admin/learningObject/${row.id}`}>
                                                <IconButton onClick={() => {
                                                    currPage = 0

                                                }}>
                                                    <VisibilityIcon style={{ fill: "#00bcd4" }} />
                                                </IconButton>
                                            </Link>
                                        </StyledTableCell>
                                        <StyledTableCell align="right">
                                            {isLoadingToDelete === index ? (
                                                <CircularProgress size={24} color="primary" />
                                            ) : (
                                                    <IconButton
                                                        onClick={() => {
                                                            HandleStateAlertDialog(index);
                                                            HandleStateCircularProgress(index);
                                                        }}
                                                    >
                                                        <DeleteIcon style={{ fill: "#FF0000" }} />
                                                    </IconButton>
                                                )}
                                        </StyledTableCell>
                                    </StyledTableRow>
                                )
                        )}
                    </TableBody>
                </TableData>
                {/************** End of display data in table **************/}

                {/* This alert will be displayed if the user click to delete an institution */}
                <AlertDialog
                    open={openAlertDialog}
                    OnDelete={DeleteHandler}
                    deleteItem={deleteItem}
                    HandleClose={() => {
                        setOpenAlertDialog(false);
                        HandleStateCircularProgress(null);
                    }}
                />
            </div>
        )
    }
};

export default EducationalObjects;
