/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useEffect, useState, useContext } from 'react'
import moment from 'moment';
//imports from local files 
import TableData from '../../../Components/Components/Table';
import SnackBar from '../../../../Components/SnackbarComponent';
import Unauthorized from '../../../Components/Components/Unauthorized';
import { Url, EditFilter } from '../../../Filters';
import { Store } from '../../../../Store';
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import { getRequest, putRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
//imports from material ui 
import { withStyles } from '@material-ui/core/styles';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';
import { Button, Typography, Paper, Grid } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import AddRoundedIcon from '@material-ui/icons/AddRounded';
import UpdateRoundedIcon from '@material-ui/icons/UpdateRounded';
import CheckCircleRoundedIcon from '@material-ui/icons/CheckCircleRounded';
import CancelRoundedIcon from '@material-ui/icons/CancelRounded';
import Switch from '@material-ui/core/Switch';
//router
import { Link } from 'react-router-dom';

let currPage = 0;
let transformListToAsc = false;

const StyledTableCell = withStyles((theme) => ({
    head: {
        backgroundColor: theme.palette.common.black,
        color: theme.palette.common.white,
    },
    body: {
        fontSize: 14,
    },
}))(TableCell);

const StyledTableRow = withStyles((theme) => ({
    root: {
        '&:nth-of-type(odd)': {
            backgroundColor: theme.palette.action.hover,
        },
    },
}))(TableRow);

const Questions = () => {
    const { state, dispatch } = useContext(Store);

    const ADD_ONE_LENGHT = [""];
    const TOP_LABELS = ['ID', 'CRIAÇÃO EM', 'DESCRIÇÃO', 'STATUS', 'ATUALIZAÇÃO EM'] //Labels from Table  

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [items, setItems] = useState([]); //Necessary to consult the API, data 

    const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false) //controlls the state of loadind more data
    const [isUpdating, setIsUpdating] = useState(false); //controlls the state of updating data

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    //handle snack info
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    const CheckUserPermission = () => {
        let canUserEdit = false;

        if (state.userIsLoggedIn) {
            const roles = [...state.currentUser.roles];
            for (let i = 0; i < roles.length; i++)
                if (roles[i].name === 'admin')
                    canUserEdit = true;
        }
        else {
            canUserEdit = false;
        }

        return canUserEdit;
    }

    //handle load more items
    const LoadMoreItens = async (api) => {
        setIsLoadingMoreItems(true)
        getRequest(
            api,
            (data, header) => {
                const arrData = [...data]
                if (arrData.length === 0) {
                    HandleSnack('Não há mais dados para serem carregados', true, 'warning', '#FFC125')
                } else {
                    const arrItems = [...items]
                    arrItems.pop(); //Deleting the last position, that was used to display the button of load more items 
                    const arrResult = arrItems.concat(arrData)
                    setItems(arrResult.concat(ADD_ONE_LENGHT))
                }
                setIsLoadingMoreItems(false)
            },
            (error) => {
                HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                setIsLoadingMoreItems(false)
            }
        )
    }

    // handle update list data
    const UpdateHandler = async (api) => {
        setIsUpdating(true)
        getRequest(
            api,
            (data, header) => {
                HandleSnack('A lista de dados foi atualizada', true, 'success', '#228B22')
                const arrData = [...data]
                setItems(arrData.concat(ADD_ONE_LENGHT))
                setIsUpdating(false)
            },
            (error) => {
                HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                setIsUpdating(false)
            }
        )
    }

    const InvertList = async () => {
        transformListToAsc = !transformListToAsc
        currPage = 0
        if (transformListToAsc) {
            getRequest(
                Url('questions', '', `${currPage}`, 'ASC'),
                (data, header) => {
                    const arrData = [...data]
                    setItems(arrData.concat(ADD_ONE_LENGHT))
                },
                () => {
                    HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                }
            )
        } else {
            getRequest(
                Url('questions', '', `${currPage}`, 'DESC'),
                (data, header) => {
                    const arrData = [...data]
                    setItems(arrData.concat(ADD_ONE_LENGHT))
                },
                (error) => {
                    HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                }
            )
        }
    }

    const handleChange = async (index, status) => {
        const id = items[index].id;
        const description = items[index].description;
        if (status === 'active') {
            const body = {
                "question": {
                    "description": description,
                    "status": "inactive"
                }
            }
            putRequest(
                EditFilter('questions', id),
                body,
                (data) => {
                    if (data.errors)
                        HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                    else {
                        currPage = 0
                        transformListToAsc = false
                        UpdateHandler(Url('questions', '', `${currPage}`, 'DESC'))
                    }
                },
                (error) => {
                    HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                }
            )
        } else {
            const body = {
                "question": {
                    "description": description,
                    "status": "active"
                }
            }
            putRequest(
                EditFilter('questions', id),
                body,
                (data) => {
                    if (data.errors)
                        HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                    else {
                        currPage = 0
                        transformListToAsc = false
                        UpdateHandler(Url('questions', '', `${currPage}`, 'DESC'))
                    }
                },
                (error) => {
                    HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                }
            )
        }
    }

    const DisplayDate = (date) => {
        const convertedData = moment.utc(date);
        return moment(convertedData)
            .format("LLL")
            .toString();
    };

    //getting data from server
    useEffect(() => {
        getRequest(
            Url('questions', '', `${currPage}`, 'DESC'),
            (data, header) => {
                setIsLoaded(true);
                setItems(data.concat(ADD_ONE_LENGHT));
            },
            (error) => {
                setError(true)
            }
        )
    }, []);


    if (error) {
        return <div>Error: {error.message}</div>;
    } else if (!isLoaded) {
        return <LoadingSpinner text="Carregando..." />
    } else if (CheckUserPermission()) {
        return (
            <>
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() => setSnackInfo({
                        message: '',
                        icon: '',
                        open: false,
                        color: ''
                    })}
                />

                <Paper style={{ padding: '1em' }}>
                    <Grid container spacing={3} direction="row" alignItems="center">
                        <Grid item xs={6}>
                            <Typography variant="h4">
                                Perguntas da curadoria
                            </Typography>
                        </Grid>
                        <Grid
                            item
                            xs={6}
                        >
                            <Grid container justify="flex-end" spacing={3}>
                                <Grid item>
                                    <Button
                                        variant="contained"
                                        color="secondary"
                                        disabled={isUpdating}
                                        onClick={() => {
                                            currPage = 0
                                            transformListToAsc = false
                                            UpdateHandler(Url('questions', '', `${currPage}`, 'DESC'))
                                        }}
                                        startIcon={<UpdateRoundedIcon />}
                                    >
                                        {
                                            isUpdating ? <CircularProgress size={24} /> : 'Atualizar'
                                        }
                                    </Button>
                                </Grid>
                                <Grid item>

                                    <Link style={{ textDecoration: 'none' }} to={'/admin/CreateQuestion'}>
                                        <Button
                                            variant="contained"
                                            color="secondary"
                                            startIcon={<AddRoundedIcon />}
                                            onClick={() => { currPage = 0 }}
                                        >
                                            Novo
                                        </Button>
                                    </Link>
                                </Grid>
                            </Grid>
                        </Grid>
                    </Grid>
                </Paper>

                <div style={{ height: '2em' }}></div>

                <TableData
                    top={TOP_LABELS}
                    onIconPressed={InvertList}
                >
                    <TableBody>
                        {items.map((row, index) => (
                            index === items.length - 1 ?
                                <StyledTableRow key={index}>
                                    {/* Button to load more data */}
                                    <StyledTableCell>
                                        <Button
                                            color='primary'
                                            variant='text'
                                            // disabled={isLoadingMoreItems}
                                            startIcon={<AddRoundedIcon />}
                                            disabled={isLoadingMoreItems}
                                            onClick={() => {
                                                currPage++
                                                if (transformListToAsc) {
                                                    LoadMoreItens(Url('questions', '', `${currPage}`, 'ASC'))
                                                } else {
                                                    LoadMoreItens(Url('questions', '', `${currPage}`, 'DESC'))
                                                }
                                            }}
                                        >
                                            {
                                                isLoadingMoreItems ? <CircularProgress size={24} /> : 'Carregar mais itens'
                                            }
                                        </Button>
                                    </StyledTableCell>
                                </StyledTableRow>

                                :

                                <StyledTableRow key={index}>
                                    <StyledTableCell component="th" scope="row">{row.id}</StyledTableCell>
                                    <StyledTableCell align="right">{DisplayDate(row.created_at)}</StyledTableCell>
                                    <StyledTableCell align="right">{row.description}</StyledTableCell>
                                    <StyledTableCell align="right">
                                        {
                                            row.status === 'active' ?
                                                <Grid container direction='row'>
                                                    <Grid item>
                                                        <CheckCircleRoundedIcon style={{ fill: '#3CB371' }} />

                                                        <Switch
                                                            checked={true}
                                                            onChange={() => handleChange(index, row.status)}
                                                            name="checkedB"
                                                            color="primary"
                                                        />
                                                    </Grid>
                                                </Grid>

                                                :

                                                <Grid container justify='flex-end' alignItems='center' direction='row'>
                                                    <Grid item>
                                                        <CancelRoundedIcon style={{ fill: '#FA8072' }} />
                                                    </Grid>

                                                    <Grid item>
                                                        <Switch
                                                            checked={false}
                                                            onChange={() => handleChange(index, row.status)}
                                                            name="checkedB"
                                                            color="primary"
                                                        />
                                                    </Grid>
                                                </Grid>
                                        }
                                    </StyledTableCell>
                                    <StyledTableCell align="right">{DisplayDate(row.updated_at)}</StyledTableCell>
                                </StyledTableRow>
                        ))}
                    </TableBody>
                </TableData>
            </>
        );
    } else return <Unauthorized />
}
export default Questions;